#property copyright " Bernd Kreuss" 
#property indicator_chart_window 

#property indicator_buffers 3
#property indicator_color1 Red
#property indicator_width1 1 
#property indicator_color2 Yellow
#property indicator_width2 1
#property indicator_color3 Yellow
#property indicator_width3 1 

#include <common_functions.mqh>
#include <mt4R.mqh>
// Eventhandler
#include <eventhandler.mqh>

extern string RTermPath_desc="Commandline to call RTERM.EXE";
extern string RTermPath="C:\Apps\R\R-2.15.3\bin\i386\Rterm.exe --no-save";
//
extern bool saveWS=false;
extern string RWorkPath_desc="Where are the R-Snapshots to be saved?";
extern string RWorkPath="C:/RData/";
//
extern string retro_desc="How many bars into the past to begin a prediction";
extern int retro=0;
//
#define PM_OPEN 0
#define PM_HIGH 1
#define PM_LOW 2
#define PM_CLOSE 3
#define PM_TYPICAL 4
#define PM_MEDIAN 5
#define PM_WEIGHTED 6
extern string priceModel_desc1="Open=0, High=1, Low=2, Close=3,";
extern string priceModel_desc2="Typical=4, Median=5, Weighted=6";
extern string priceModel_desc3="Default: 4 (Close)";
extern int priceModel=4;
//
extern string back_desc = "How many bars into the past from retro to sample the data";
extern int back = 1000;
//
extern string ahead_desc="How many bars to predict";
extern int ahead = 150;
//
extern string Rmethod_desc="Allowed: yule-walker, burg, ols, mle, yw";
extern string Rmethod="ols";
static int curPeriod=0;
//
extern string order_desc="Filter order of the autoregression";
extern int order = 200;

int R;
double buf_prediction[];
double buf_upper[];
double buf_lower[];

// double hist_r[][6];
MqlRates hist_r[];


string 	CR="\r\n";
string PREDICTION_COMMAND;

int init(){
   ArrayResize(buf_prediction, ahead);
	ArrayResize(buf_upper, ahead);
	ArrayResize(buf_lower, ahead);
	SetIndexBuffer(0, buf_prediction);
	SetIndexStyle(0, DRAW_LINE);
	SetIndexShift(0, ahead-retro);
	//
	SetIndexBuffer(1, buf_upper);
	SetIndexStyle(1, DRAW_LINE);
	SetIndexShift(1, ahead-retro);
	//
	SetIndexBuffer(2, buf_lower);
	SetIndexStyle(2, DRAW_LINE);
	SetIndexShift(2, ahead-retro);
	//
	// That's the command cluster which calculates the predictions
	PREDICTION_COMMAND= 
		  "z <- head(tail(hist_r, samples+retro), samples)" + CR
		+ "z <- as.xts(z[,2:6], order.by=as.POSIXct(z$Timestamp, origin='1970-01-01 00:00', tz='UTC'))" + CR
		+ "if (priceModel==PM_OPEN) { hist <- z$Open } " + CR
		+ "if (priceModel==PM_HIGH) { hist <- z$High } " + CR
		+ "if (priceModel==PM_LOW) { hist <- z$Low } " + CR
		+ "if (priceModel==PM_CLOSE) { hist <- z$Close } " + CR
		+ "if (priceModel==PM_TYPICAL) { hist <- (z$High + z$Low + z$Close) / 3 } " + CR
		+ "if (priceModel==PM_MEDIAN) { hist <- (z$High + z$Low) / 2 } " + CR
		+ "if (priceModel==PM_WEIGHTED) { hist <- (z$High + z$Low + 2*z$Close) / 4 } " + CR
		+ "h <- as.xts(hist)" + CR
		+ "model <- ar(h, aic=FALSE, order=ord, method='" + Rmethod  + "')" + CR
		+ "p <- predict(model, n.ahead=ahead)" + CR
		+ "pred <- p$pred" + CR
		+ "ub <- last(z$Close) + p$se" + CR
		+ "lb <- last(z$Close) - p$se";
	curPeriod=Period();
	R = RInit(RTermPath, 2);
	//
	// Assign const values ...
	RAssignInteger(R, "PM_OPEN", PM_OPEN);
	RAssignInteger(R, "PM_HIGH", PM_HIGH);
	RAssignInteger(R, "PM_LOW", PM_LOW);
	RAssignInteger(R, "PM_CLOSE", PM_CLOSE);
	RAssignInteger(R, "PM_TYPICAL", PM_TYPICAL);
	RAssignInteger(R, "PM_MEDIAN", PM_MEDIAN);
	RAssignInteger(R, "PM_WEIGHTED", PM_WEIGHTED);
	//
	// Assign the parameters for the calculation
	RAssignInteger(R, "ord", order);
	RAssignInteger(R, "ahead", ahead);
	RAssignInteger(R, "retro", retro);
	RAssignInteger(R, "samples", back);
	RAssignInteger(R, "priceModel", priceModel);
	//
	// Prepare the packages needed ...
	RExecute(R, 
		  "pkgs <- c('quantmod')" + CR
		+ "lapply(pkgs, require, character.only=T)"
	);
	
	Comment("$Rev: 672 $ Retrace " + retro + " bars, history: " + back + " bars, method: " + Rmethod + " , order: " + order + 
	", Datapath: " + RWorkPath);
	// Initialize Eventhandler to provide onBar and onTick Events.
	events_initEvents();
	doRDraw();
	if (saveWS) {
		RExecute(R, 
			  "setwd('" + RWorkPath + "')"+CR
			+ "save.image(file='"+ createFileName() + "')"
		);
	}
	return(0);
}

int deinit(){
	Comment("");
	RDeinit(R);
	return(0);
}

int start(){
	// Perform eventhandler calls.
	events_doEvents();
	return(0);   
}

void events_onBar(int pPeriod) {
	switch (pPeriod) {
	case PERIOD_M1:
		// Print("onBar: working on M1...");
		doRDraw();
		break;
	case PERIOD_M5:
		// Print("onBar: working on M5...");
		if (saveWS) {
			RExecute(R,"save.image(file='"+ createFileName() + "')");
		}
		break;
	case PERIOD_M15:
		break;
	case PERIOD_M30:
		break;
	case PERIOD_H1:
		break;
	case PERIOD_H4:
		break;
	case PERIOD_D1:
		break;
	case PERIOD_W1:
		break;
	default:
		break;
	}
	return;
}

void events_onTick() {
	return;
}

void doRDraw() {
	static int curPeriod=0;
	// Hier stehen die Vorhersagen...
	double pred[];
	double ub[];
	double lb[];
	int i;
	// Switching of timeframes does funny things :-)
	if (Period()!=curPeriod) {
		curPeriod=Period();
		doRDraw();
	}
	// make a (new) prediction
	// Return buffers
	ArrayResize(pred, ahead);
	ArrayResize(ub, ahead);
	ArrayResize(lb, ahead);
	// 
	int cnt=ArrayCopyRates(hist_r, NULL, 0);
	while (GetLastError()==4066) {
		Print("init: ArrayCopyRates barfed... retrying");
		Sleep(1000);
		cnt=ArrayCopyRates(hist_r, NULL, 0);
	}
	log("doRDraw: Bars=", DoubleToStr( iBars(NULL,0), 0), " vs. cnt=", DoubleToStr(cnt, 0) );
	// Prepare the data for the indicator ...
	RAssignOHLC(R, "hist_r", hist_r,  iBars(NULL,0));
	// Do the calculations ...
	RExecute(R, PREDICTION_COMMAND);
	RGetVector(R, "pred", pred, ahead);
	RGetVector(R, "ub", ub, ahead);
	RGetVector(R, "lb", lb, ahead);
	int j;
	for (i=0; i<ahead; i++){
		j=ahead - i - 1;
		buf_prediction[i] = pred[j];
		buf_upper[i] = ub[j];
		buf_lower[i] = lb[j];
	}
}


/**
creates a filename for the RData export file.
*/
string createFileName() {
	string name=
		WindowExpertName() + "_" + 
		Symbol() + "_" + 
		Period() + 
		".RData";
	return(name);
}
